* Open a specified workbook, and if it's stored in a format
* later than Excel 95, resave it in the older format. 
* If necessary, break it into multiple workbooks.
* Several caveats: 
*   1) Works only on the active sheet of the workbook.
*   2) If it's necessary to break the worksheet up, this code
*      assumes there are no formulas working across multiple rows.
* Return the number of resulting workbooks. 
* Return 0 if no change is needed.
* Return a negative value to indicate a problem.

LPARAMETERS cFileWithPath

LOCAL oExcel, oWorkbook, nWorkbookCount, cBaseName, cPath

TRY 
	oExcel = CREATEOBJECT("Excel.Application")
CATCH
	oExcel = .null.
ENDTRY

IF ISNULL(m.oExcel)
	RETURN -1
ENDIF 

TRY 
	oWorkbook = oExcel.Workbooks.Open(m.cFileWithPath)
CATCH 
	oWorkbook = .null.
ENDTRY

IF ISNULL(m.oWorkbook)
	RETURN -1
ENDIF

* If we get this far, we've opened Excel and the workbook. 
* Now figure out whether we need to convert.
cPath = JUSTPATH(m.cFileWithPath)
cBaseName = JUSTSTEM(m.cFileWithPath) + "XL5"

DO CASE 
CASE oWorkbook.FileFormat <= 39 && Excel 5 or earlier
	* Nothing to do. 
	nWorkbookCount = 0

CASE oWorkbook.ActiveSheet.UsedRange.Rows.Count <= 16384
	* Just save as.
	cFileName = FORCEPATH(m.cBaseName, m.cPath) && extension is automatic
	oWorkbook.SaveAs(m.cFileName, 39)
	nWorkbookCount = 1

OTHERWISE
	* Need to break up into multiple workbooks.
	LOCAL nTotalRows, nSheetsNeeded, nSheet, oNewBook, nLastColumn, oRangeToCopy
	WITH oWorkbook.ActiveSheet
	
		nTotalRows = .UsedRange.Rows.Count
		nSheetsNeeded = CEILING(m.nTotalRows/16384)
		
		nLastColumn = .UsedRange.Columns.Count
		
		FOR nSheet = 1 TO m.nSheetsNeeded
			oNewBook = oExcel.Workbooks.Add()
			oRange = .Range(.Cells((m.nSheet-1) * 16384 + 1, 1), .Cells(m.nSheet * 16384, m.nLastColumn))
			oRange.Copy(oNewBook.ActiveSheet.Range("A1"))
			
			cFileName = FORCEPATH(m.cBaseName + "_" + TRANSFORM(m.nSheet), m.cPath)
			oNewBook.SaveAs(m.cFileName, 39)
			oNewBook.Close()
		ENDFOR
		
	ENDWITH
	
	nWorkBookCount = m.nSheetsNeeded
ENDCASE

oWorkBook.Close()
oExcel.Quit()

RETURN m.nWorkBookCount