*==============================================================================
* Class:				OfficeHandler
* Based On:				Session
* Purpose:				Implements Office server events
* Author:				Tamar E. Granor
* Copyright:			(c) 2010 Tamar E. Granor, Ph.D.
* Last revision:		9/2/2010
* Include file:			None
*==============================================================================

*******************************************************************************
DEFINE CLASS OfficeHandler AS session OLEPUBLIC

	* Word
    IMPLEMENTS ApplicationEvents4 in "Word.Application"
    * Excel
    IMPLEMENTS AppEvents IN "Excel.Application"
    * PowerPoint
    IMPLEMENTS EApplication IN "PowerPoint.Application"


	PROTECTED cLogTable, cLogPath, cLogFullPath
	cLogTable = "OfficeLogEventHandler.DBF"

	PROCEDURE Init
	
	This.cLogPath = SYS(2023)
	This.cLogFullPath = FORCEPATH(This.cLogTable, ;
    	                          This.cLogPath)

	* Make sure the log table exists
	IF NOT FILE(This.cLogFullPath)
    	CREATE TABLE (This.cLogFullPath) ;
          (cDocument C(150), cAction C(12), tModified T)
        CLOSE TABLES 
	ENDIF
	
	SET EXCLUSIVE OFF 
	RETURN 

	*****
	* Begin Word events
	*****
	PROCEDURE ApplicationEvents4_DocumentOpen(Doc AS VARIANT) AS VOID

	* Log open
	INSERT INTO (This.cLogFullPath) ;
         VALUES (Doc.FullName, "Opened", DATETIME())

	RETURN .t.

	ENDPROC

	PROCEDURE ApplicationEvents4_DocumentBeforeClose(Doc AS VARIANT, Cancel AS LOGICAL) AS VOID

	* Log close
	INSERT INTO (This.cLogFullPath) ;
         VALUES (Doc.FullName, "Closed", DATETIME())

	RETURN .t.

	ENDPROC

	PROCEDURE ApplicationEvents4_DocumentBeforeSave(Doc AS VARIANT, SaveAsUI AS LOGICAL, Cancel AS LOGICAL) AS VOID

	* Log save
	INSERT INTO (This.cLogFullPath) ;
         VALUES (Doc.FullName, "Saved", DATETIME())

	RETURN .t.

	ENDPROC

	PROCEDURE ApplicationEvents4_NewDocument(Doc AS VARIANT) AS VOID

	* Log create
	INSERT INTO (This.cLogFullPath) ;
         VALUES (Doc.FullName, "Created", DATETIME())

	RETURN .t.

	ENDPROC

	*****
	* Not using these events of Word 
	*****
	PROCEDURE ApplicationEvents4_DocumentBeforePrint(Doc AS VARIANT, Cancel AS LOGICAL) AS VOID
		* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_Quit() AS VOID
		* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_DocumentChange() AS VOID
		* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_WindowActivate(Doc AS VARIANT, Wn AS VARIANT) AS VOID
		* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_WindowDeactivate(Doc AS VARIANT, Wn AS VARIANT) AS VOID
		* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_WindowSelectionChange(Sel AS VARIANT) AS VOID
		* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_WindowBeforeRightClick(Sel AS VARIANT, Cancel AS LOGICAL) AS VOID
		* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_WindowBeforeDoubleClick(Sel AS VARIANT, Cancel AS LOGICAL) AS VOID
		* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_EPostagePropertyDialog(Doc AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_EPostageInsert(Doc AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_MailMergeAfterMerge(Doc AS VARIANT, DocResult AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_MailMergeAfterRecordMerge(Doc AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_MailMergeBeforeMerge(Doc AS VARIANT, StartRecord AS Number, EndRecord AS Number, Cancel AS LOGICAL @) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_MailMergeBeforeRecordMerge(Doc AS VARIANT, Cancel AS LOGICAL @) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_MailMergeDataSourceLoad(Doc AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_MailMergeDataSourceValidate(Doc AS VARIANT, Handled AS LOGICAL) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_MailMergeWizardSendToCustom(Doc AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_MailMergeWizardStateChange(Doc AS VARIANT, FromState AS Integer, ToState AS Integer, Handled AS LOGICAL) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_WindowSize(Doc AS VARIANT, Wn AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_XMLSelectionChange(Sel AS VARIANT, OldXMLNode AS VARIANT, NewXMLNode AS VARIANT, Reason AS Number) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_XMLValidationError(XMLNode AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_DocumentSync(Doc AS VARIANT, SyncEventType AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_EPostageInsertEx(Doc AS VARIANT, cpDeliveryAddrStart AS Integer, cpDeliveryAddrEnd AS Integer, cpReturnAddrStart AS Integer, cpReturnAddrEnd AS Integer, xaWidth AS Integer, yaHeight AS Integer, bstrPrinterName AS STRING, bstrPaperFeed AS STRING, fPrint AS LOGICAL, fCancel AS LOGICAL @) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_MailMergeDataSourceValidate2(Doc AS VARIANT, Handled AS LOGICAL @) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_ProtectedViewWindowOpen(PvWindow AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_ProtectedViewWindowBeforeEdit(PvWindow AS VARIANT, Cancel AS LOGICAL @) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_ProtectedViewWindowBeforeClose(PvWindow AS VARIANT, CloseReason AS Integer, Cancel AS LOGICAL @) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_ProtectedViewWindowSize(PvWindow AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_ProtectedViewWindowActivate(PvWindow AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE ApplicationEvents4_ProtectedViewWindowDeactivate(PvWindow AS VARIANT) AS VOID
	* add user code here
	ENDPROC
	

	*****
	* Begin Excel events
	*****

	PROCEDURE AppEvents_NewWorkbook(Wb AS VARIANT) AS VOID

	* Log create
	INSERT INTO (This.cLogFullPath) ;
         VALUES (Wb.FullName, "Created", DATETIME())

	RETURN .t.

	ENDPROC

	PROCEDURE AppEvents_WorkbookOpen(Wb AS VARIANT) AS VOID

	* Log open
	INSERT INTO (This.cLogFullPath) ;
         VALUES (Wb.FullName, "Opened", DATETIME())

	RETURN .t.

	ENDPROC

	PROCEDURE AppEvents_WorkbookBeforeClose(Wb AS VARIANT, Cancel AS LOGICAL) AS VOID

	* Log close
	INSERT INTO (This.cLogFullPath) ;
         VALUES (Wb.FullName, "Closed", DATETIME())

	RETURN .t.

	ENDPROC

	PROCEDURE AppEvents_WorkbookBeforeSave(Wb AS VARIANT, SaveAsUI AS LOGICAL, Cancel AS LOGICAL) AS VOID

	* Log save
	INSERT INTO (This.cLogFullPath) ;
         VALUES (Wb.FullName, "Saved", DATETIME())

	RETURN .t.

	ENDPROC

	*****
	* Not using these events of Excel
	*****
	PROCEDURE AppEvents_WorkbookActivate(Wb AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_WorkbookDeactivate(Wb AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_SheetSelectionChange(Sh AS VARIANT, Target AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_SheetBeforeDoubleClick(Sh AS VARIANT, Target AS VARIANT, Cancel AS LOGICAL) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_SheetBeforeRightClick(Sh AS VARIANT, Target AS VARIANT, Cancel AS LOGICAL) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_SheetActivate(Sh AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_SheetDeactivate(Sh AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_SheetCalculate(Sh AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_SheetChange(Sh AS VARIANT, Target AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_WorkbookBeforePrint(Wb AS VARIANT, Cancel AS LOGICAL) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_WorkbookNewSheet(Wb AS VARIANT, Sh AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_WorkbookAddinInstall(Wb AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_WorkbookAddinUninstall(Wb AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_WindowResize(Wb AS VARIANT, Wn AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_WindowActivate(Wb AS VARIANT, Wn AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_WindowDeactivate(Wb AS VARIANT, Wn AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_SheetFollowHyperlink(Sh AS VARIANT, Target AS VARIANT) AS VOID
	* add user code here
	ENDPROC
	
	* Excel 2002 and later
	PROCEDURE AppEvents_SheetPivotTableUpdate(Sh AS VARIANT, Target AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_WorkbookPivotTableCloseConnection(Wb AS VARIANT, Target AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_WorkbookPivotTableOpenConnection(Wb AS VARIANT, Target AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_WorkbookSync(Wb AS VARIANT, SyncEventType AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_WorkbookBeforeXmlImport(Wb AS VARIANT, Map AS VARIANT, Url AS STRING, IsRefresh AS LOGICAL, Cancel AS LOGICAL @) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_WorkbookAfterXmlImport(Wb AS VARIANT, Map AS VARIANT, IsRefresh AS LOGICAL, Result AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_WorkbookBeforeXmlExport(Wb AS VARIANT, Map AS VARIANT, Url AS STRING, Cancel AS LOGICAL @) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_WorkbookAfterXmlExport(Wb AS VARIANT, Map AS VARIANT, Url AS STRING, Result AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_WorkbookRowsetComplete(Wb AS VARIANT, Description AS STRING, Sheet AS STRING, Success AS LOGICAL) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_AfterCalculate() AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_SheetPivotTableAfterValueChange(Sh AS VARIANT, TargetPivotTable AS VARIANT, TargetRange AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_SheetPivotTableBeforeAllocateChanges(Sh AS VARIANT, TargetPivotTable AS VARIANT, ValueChangeStart AS Number, ValueChangeEnd AS Number, Cancel AS LOGICAL @) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_SheetPivotTableBeforeCommitChanges(Sh AS VARIANT, TargetPivotTable AS VARIANT, ValueChangeStart AS Number, ValueChangeEnd AS Number, Cancel AS LOGICAL @) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_SheetPivotTableBeforeDiscardChanges(Sh AS VARIANT, TargetPivotTable AS VARIANT, ValueChangeStart AS Number, ValueChangeEnd AS Number) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_ProtectedViewWindowOpen(Pvw AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_ProtectedViewWindowBeforeEdit(Pvw AS VARIANT, Cancel AS LOGICAL @) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_ProtectedViewWindowBeforeClose(Pvw AS VARIANT, Reason AS VARIANT, Cancel AS LOGICAL @) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_ProtectedViewWindowResize(Pvw AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_ProtectedViewWindowActivate(Pvw AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_ProtectedViewWindowDeactivate(Pvw AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_WorkbookAfterSave(Wb AS VARIANT, Success AS LOGICAL) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE AppEvents_WorkbookNewChart(Wb AS VARIANT, Ch AS VARIANT) AS VOID
	* add user code here
	ENDPROC



	*****
	* Begin PowerPoint events
	*****
	PROCEDURE EApplication_PresentationClose(Pres AS VARIANT) AS VOID
	* Log close 
	INSERT INTO (This.cLogFullPath) ;
         VALUES (Pres.FullName, "Closed", DATETIME())

	RETURN .t.
	ENDPROC

	PROCEDURE EApplication_PresentationSave(Pres AS VARIANT) AS VOID
	* Log save 
	INSERT INTO (This.cLogFullPath) ;
         VALUES (Pres.FullName, "Saved", DATETIME())

	RETURN .t.
	ENDPROC

	PROCEDURE EApplication_PresentationOpen(Pres AS VARIANT) AS VOID
	* Log open
	INSERT INTO (This.cLogFullPath) ;
         VALUES (Pres.FullName, "Opened", DATETIME())

	RETURN .t.
	ENDPROC

	PROCEDURE EApplication_NewPresentation(Pres AS VARIANT) AS VOID
	* Log creation
	INSERT INTO (This.cLogFullPath) ;
         VALUES (Pres.FullName, "Created", DATETIME())

	RETURN .t.
	ENDPROC

	*****
	* Not using these events of PowerPoint
	*****
	PROCEDURE EApplication_WindowSelectionChange(Sel AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_WindowBeforeRightClick(Sel AS VARIANT, Cancel AS LOGICAL @) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_WindowBeforeDoubleClick(Sel AS VARIANT, Cancel AS LOGICAL @) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_PresentationNewSlide(Sld AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_WindowActivate(Pres AS VARIANT, Wn AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_WindowDeactivate(Pres AS VARIANT, Wn AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_SlideShowBegin(Wn AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_SlideShowNextBuild(Wn AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_SlideShowNextSlide(Wn AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_SlideShowEnd(Pres AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_PresentationPrint(Pres AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	* PowerPoint 2002 and later
	PROCEDURE EApplication_SlideSelectionChanged(SldRange AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_ColorSchemeChanged(SldRange AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_PresentationBeforeSave(Pres AS VARIANT, Cancel AS LOGICAL @) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_SlideShowNextClick(Wn AS VARIANT, nEffect AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_AfterNewPresentation(Pres AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_AfterPresentationOpen(Pres AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_PresentationSync(Pres AS VARIANT, SyncEventType AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_SlideShowOnNext(Wn AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_SlideShowOnPrevious(Wn AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_PresentationBeforeClose(Pres AS VARIANT, Cancel AS LOGICAL @) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_ProtectedViewWindowOpen(ProtViewWindow AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_ProtectedViewWindowBeforeEdit(ProtViewWindow AS VARIANT, Cancel AS LOGICAL @) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_ProtectedViewWindowBeforeClose(ProtViewWindow AS VARIANT, ProtectedViewCloseReason AS VARIANT, Cancel AS LOGICAL @) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_ProtectedViewWindowActivate(ProtViewWindow AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_ProtectedViewWindowDeactivate(ProtViewWindow AS VARIANT) AS VOID
	* add user code here
	ENDPROC

	PROCEDURE EApplication_PresentationCloseFinal(Pres AS VARIANT) AS VOID
	* add user code here
	ENDPROC
	
ENDDEFINE

